@@ -4,7 +4,7 @@
 	 ꥨ ͥåȥңУǥ <ͥåȥط饤֥>
 	 Version 0.001, October 2000
 	 ------------------------------------------------------------
-	 Copyright (C) 2000ǯ naoki kuwamori<naoki@kuwamori.com>
+	 Copyright (C) 2000,2001,2002ǯ naoki kuwamori<naoki@kuwamori.com>
 
 	 ܥץϥե꡼եȥǤʤϡFree Software Foundation
 	 ɽGNU ̸ͭѵΡ֥С󣲡װϤʹߤγƥС
@@ -17,7 +17,7 @@
 
 	Laurier Net work RPG server<game network library>
 	Version 0.001, October 2000
-	Copyright (C) 2000 Naoki Kuwamori <naoki@kuwamori.com>
+	Copyright (C) 2000,2001,2002 Naoki Kuwamori <naoki@kuwamori.com>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -917,6 +917,10 @@ int read_packet(int FD	 , int requestlen , int *read_len , char * buf , int bufs
 	int ret;					/**/
 	int lessofbuf = 0;
 
+	if (requestlen==0){
+		*read_len=0;
+		return LIBNETOK;
+	}
 	stat = LIBNETOK;
 	*read_len = 0;
 	rpglog(RPGDEBUGLEVEL , "libnet:main loop\n");
@@ -1140,7 +1144,7 @@ int RPG_Poll( char * command , char * data , int buf_length , int *length , int
 				}
 				/*׵Ĺ*/
 				requestlen = ntohl(packet_head.length);
-				if (requestlen <= 0){
+				if (requestlen < 0){
 					/*ͤλϥ顼֤*/
 					return LIBNETNOREQLEN;
 				}
@@ -2029,6 +2033,101 @@ errtable errmessage[MAXERRNOM] = {
 };
 
 
+/*
+ *	CSVɤ߹ߴؿ
+ *	yamakawa yamakawa@users.sourceforge.jp
+ * Copyright (C) 2002
+ * */
+#if 0
+/* CSVɤ߹ */
+
+typedef struct {
+    int iOffset;        /* ¤ΤγФؤΥեå(ñ̥Х) */
+    void* (*pfConv)( void* pMem, char* pcCSV, int iParam ); /* CSV -> Ѵؿ */
+    int iParam;         /* pfConvؿ裳 */
+    int iRepeat;        /* ֤ */
+    
+} CCSVTemplate;
+#define CSVTRECORD( iOffset, pfFunc, iParam ) { iOffset, pfFunc, iParam, 1 }
+#define CSVTRECORD_ARRAY( iOffset, pfFunc, iParam, iRepeat ) { iOffset, pfFunc, iParam, iRepeat }
+#define EndOfCSVTemplate { 0, NULL, 0, 0 }
+#define IsEndOfCSVTemplate( p ) ( (p)->pfConv == (void*(*)(void*,char*,int))NULL )
+
+/* ƥץ졼Ȥǻꤵ줿CSVΥǡߤ򤹤롣 */
+/* »:CSV1ܤ1024ХȰʲǤ뤳 */
+extern int/**/ GetCSV(
+    void* pMem,
+    char* pcCSV,
+    CCSVTemplate* pTemplate
+);  
+    
+/* ƥץ졼Ȥǻꤵ줿CSVΥǡߤ򤹤롣(ƥݥХåեʬѰդ) */
+extern int/**/ GetCSV_tmpbuff(
+    void* pMem,
+    char* pcCSV,
+    CCSVTemplate* pTemplate,
+    char* pcTmp,    /* CSVɤ߹߻˻Ѥ */
+    int sizeTmp
+);
+
+/* CSV -> Ѵؿ(CCSVTemplate¤ΤΡpfConv˻ꤹؿ) */
+/* ̤δؿͤȤƼΤ褦ʤΤꤵƤ롣 */
+/* : */
+/*   pMem: */
+/*   pCSV:CSV */
+/*   iParam:ƴؿͭǽʥѥ᡼ */
+/*  */
+/*   Υ(iRepeatѻΤ) */
+/*   NULLξ硢 */
+extern void* CSV_discard( void* pMem, char* pcCSV, int iParam/**/ );  /* CSVʸΤƤ */
+extern void* CSV_int( void* pMem, char* pcCSV, int iParam/**/ );      /* CSVʸintѿȤƼ>
+ */
+extern void* CSV_short( void* pMem, char* pcCSV, int iParam/**/ );    /* CSVʸshortѿȤ>
+ */
+extern void* CSV_char( void* pMem, char* pcCSV, int iParam/**/ );     /* CSVʸcharѿȤƼ
+ */
+extern void* CSV_float( void* pMem, char* pcCSV, int iParam/**/ );    /* CSVʸfloatѿȤ>
+ */
+extern void* CSV_double( void* pMem, char* pcCSV, int iParam/**/ );   /* CSVʸdoubleѿȤ
+ */
+extern void* CSV_string( void* pMem, char* pcCSV, int iParam/*'\0'ޤʸ*/ ); /* CSVʸʸ
+˥ԡ(ü'\0') */
+extern void* CSV_memory( void* pMem, char* pcCSV, int iParam/*Хȿ*/ ); /* Τޤޥԡ */
+/* ...ޥؿ򼫺ơѤ뤳Ȥǽ (^^)v */
+
+/*Ȥ
+
+Ȥ
+typedef struct{
+	short nomber1;
+	char somestring[30];
+}testst;
+Τ褦ʹ¤Τä
+---------------file start------------------
+125,some nomber name
+345,string some thing
+--------------file end--------------------
+ʤդ뤬ä
+ƥץ졼Ȥ
+static CCSVTemplate tmpl_templateofcsvrec[] = { 
+	CSVTRECORD( offsetof( testst, nomber1),CSV_short,0), 
+	CSVTRECORD( offsetof( testst,somestring),CSV_string,30),
+	EndOfCSVTemplate
+};
+ä
+testst testval;
+ե򳫤Ƥ 
+ɤ߹ߤʤ饻åȤޤ
+while ( fgets( buff, sizeof( buff ), fp ), !feof( fp ) ) {
+	memset (&testval , 0 ,sizeof(testst));
+	if (FALSE==GetCSV( &testval, buff, tmpl_templateofcsvrec )){
+		printf("error of read cvs file\n");
+	}   
+	printf("print nom[%d] string[%s]\n", testval.nomber1 , testval.somestring);
+}
+ʴǻȤޤ
+*/
+#endif
 
 char *rpg_net_perror(int l_errno ,char ** message){
 char *ret = NULL;
@@ -2044,5 +2143,160 @@ int cnt;
 	return ret;
 }
 
+static char* EscapeCSV( char* pcTmp, char* pcCSV, int sizeTmp )
+{
+    int count = 0;
+
+    while ( *pcCSV != '\0' && *pcCSV != ',' ) {
+
+        if ( count >= sizeTmp ) {
+            /* Ѱդ줿ƥݥХåե礭򤳤硢Ԥ */
+            return NULL;
+        }
+        /* '\'(ʸ) */
+        if ( *pcCSV == '\\' ) {
+            /* '\'μʸ롣 */
+            *pcTmp++ = *( pcCSV + 1 );
+            pcCSV += 2;
+        }
+        else {
+            /* ̾ʸϤΤޤޥԡ롣 */
+            *pcTmp++ = *pcCSV++;
+        }
+        ++count;
+    }
+    *pcTmp = '\0';
+
+    return ++pcCSV;
+}
+
+/*
+ *  * ãӣʸ󤫤鹽¤Τ˥쥳ɤ򥻥åȤ */
+int GetCSV( void* pMem, char* pcCSV, CCSVTemplate* pTemplate )
+{
+    char pcTmp[ 1024 ];
+
+    return GetCSV_tmpbuff( pMem, pcCSV, pTemplate, pcTmp, sizeof( pcTmp ) );
+}
+
+/*
+ *  *  */
+int GetCSV_tmpbuff(
+    void*           pMem,
+    char*           pcCSV,
+    CCSVTemplate*   pTemplate,
+    char*           pcCSVTmp,
+    int             sizeCSVTmp )
+{
+    while ( !IsEndOfCSVTemplate(pTemplate) ) {
+
+        int     i;
+        char*   pMemTmp = (char*)pMem + pTemplate->iOffset;
+
+        for ( i = 0 ; i < pTemplate->iRepeat ; ++i ) {
+            pcCSV = EscapeCSV( pcCSVTmp, pcCSV, sizeCSVTmp );
+            if ( !pcCSV ) {
+                return FALSE;
+            }
+            pMemTmp = pTemplate->pfConv( (char*)pMemTmp, pcCSVTmp,
+                pTemplate->iParam );
+            if ( !pMemTmp ) {
+                return FALSE;
+            }
+        }
+        ++pTemplate;
+    }
+
+    return TRUE;
+}
+
+/*
+ *  * ɤФȤ˻Ѥޤ */
+void* CSV_discard(
+    void*   pMem/*ߡ*/,
+    char*   pcCSV/*ߡ*/,
+    int     iParam/*ߡ*/
+)
+{
+    return pMem;
+}
+
+/*
+ *  * ѴСintegerλ˻Ѥޤ */
+void* CSV_int( void* pMem, char* pcCSV, int iParam/**/ )
+{
+
+    *(int*)pMem = atoi( pcCSV );
+    return (int*)pMem + 1;
+}
+
+void* CSV_uint( void* pMem, char* pcCSV, int iParam/**/ )
+{
+
+    *(int*)pMem = atoi( pcCSV );
+    return (unsigned int*)pMem + 1;
+}
+
+/*
+ *  * ѴСintegerλ˻Ѥޤ */
+void* CSV_short( void* pMem, char* pcCSV, int iParam/**/ )
+{
+
+    *(short*)pMem = atoi( pcCSV );
+    return (short*)pMem + 1;
+}
+
+void* CSV_ushort( void* pMem, char* pcCSV, int iParam/**/ )
+{
+
+    *(short*)pMem = atoi( pcCSV );
+    return (unsigned short*)pMem + 1;
+}
+/*
+ *  * ѴСcharλ˻Ѥޤ */
+void* CSV_char( void* pMem, char* pcCSV, int iParam/**/ )
+{
+    *(char*)pMem = atoi( pcCSV );
+    return (char*)pMem + 1;
+}
+
+void* CSV_uchar( void* pMem, char* pcCSV, int iParam/**/ )
+{
+    *(char*)pMem = atoi( pcCSV );
+    return (unsigned char*)pMem + 1;
+}
+
+/*
+ *  * ѴСʸ(NULLȥå)λ˻Ѥޤ */
+void* CSV_string( void* pMem, char* pcCSV, int iParam/*ʸ*/ )
+{
+    strncpy( pMem, pcCSV, iParam );
+    return (char*)pMem + iParam;
+}
+
+/*
+ *  * ѴСʸ(NULLȥåפʤ)λ˻Ѥޤ */
+void* CSV_memory( void* pMem, char* pcCSV, int iParam/*ʸ*/ )
+{
+    memcpy( pMem, pcCSV, iParam );
+    return (char*)pMem + iParam;
+}
+
+/*
+ *  * ѴСfloatλ˻Ѥޤ */
+void* CSV_float( void* pMem, char* pcCSV, int iParam/*ʸ*/ )
+{
+    *(float*)pMem = (float)atof( pcCSV );
+    return (float*)pMem + 1;
+}
+
+/*
+ *  * ѴСdoubleλ˻Ѥޤ */
+void* CSV_double( void* pMem, char* pcCSV, int iParam/*ʸ*/ )
+{
+    *(double*)pMem = atof( pcCSV );
+    return (double*)pMem + 1;
+}
+
 
 
