@@ -55,6 +55,8 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <sys/mman.h>
+#include <stddef.h>
+#include <netinet/in.h>
 
 /*
 #include "rpgnet.h"
@@ -84,6 +86,12 @@
 #ifndef RPG_NG
 #	define RPG_NG		0
 #endif
+#ifndef FALSE
+#define FALSE 0
+#endif
+#ifndef TRUE
+#define TRUE 1
+#endif
 
 #define LIBNET
 #define LIBNET
@@ -1170,6 +1178,67 @@ int rpg_gethash ( unsigned short * data , int len);
 char *rpg_net_perror(int l_errno ,char ** message);
 /**************************************************************************/
 
+/****************************************************************************/
+/* CSVɤ߹ 
+ *
+ * yamakawa <yamakawa@users.sourceforge.jp>
+ * */
+
+typedef struct {
+    int iOffset;        /* ¤ΤγФؤΥեå(ñ̥Х) */
+    void* (*pfConv)( void* pMem, char* pcCSV, int iParam ); /* CSV -> Ѵؿ */
+    int iParam;         /* pfConvؿ裳 */
+    int iRepeat;        /* ֤ */
+    
+} CCSVTemplate;
+#define CSVTRECORD( iOffset, pfFunc, iParam ) { iOffset, pfFunc, iParam, 1 }
+#define CSVTRECORD_ARRAY( iOffset, pfFunc, iParam, iRepeat ) { iOffset, pfFunc, iParam, iRepeat }
+#define EndOfCSVTemplate { 0, NULL, 0, 0 }
+#define IsEndOfCSVTemplate( p ) ( (p)->pfConv == (void*(*)(void*,char*,int))NULL )
+
+/* ƥץ졼Ȥǻꤵ줿CSVΥǡߤ򤹤롣 */
+/* »:CSV1ܤ1024ХȰʲǤ뤳 */
+extern int/**/ GetCSV(
+    void* pMem,
+    char* pcCSV,
+    CCSVTemplate* pTemplate
+);
+
+/* ƥץ졼Ȥǻꤵ줿CSVΥǡߤ򤹤롣(ƥݥХåեʬѰդ) */
+extern int/**/ GetCSV_tmpbuff(
+    void* pMem,
+    char* pcCSV,
+    CCSVTemplate* pTemplate,
+    char* pcTmp,    /* CSVɤ߹߻˻Ѥ */
+    int sizeTmp
+);
+
+/* CSV -> Ѵؿ(CCSVTemplate¤ΤΡpfConv˻ꤹؿ) */
+/* ̤δؿͤȤƼΤ褦ʤΤꤵƤ롣 */
+/* : */
+/*   pMem: */
+/*   pCSV:CSV */
+/*   iParam:ƴؿͭǽʥѥ᡼ */
+/*  */
+/*   Υ(iRepeatѻΤ) */
+/*   NULLξ硢 */
+extern void* CSV_discard( void* pMem, char* pcCSV, int iParam/**/ );  /* CSVʸΤƤ */
+extern void* CSV_int( void* pMem, char* pcCSV, int iParam/**/ );      /* CSVʸintѿȤƼ>
+ */
+extern void* CSV_short( void* pMem, char* pcCSV, int iParam/**/ );    /* CSVʸshortѿȤ>
+ */
+extern void* CSV_char( void* pMem, char* pcCSV, int iParam/**/ );     /* CSVʸcharѿȤƼ
+ */
+extern void* CSV_float( void* pMem, char* pcCSV, int iParam/**/ );    /* CSVʸfloatѿȤ>
+ */
+extern void* CSV_double( void* pMem, char* pcCSV, int iParam/**/ );   /* CSVʸdoubleѿȤ
+ */
+extern void* CSV_string( void* pMem, char* pcCSV, int iParam/*'\0'ޤʸ*/ ); /* CSVʸʸ
+˥ԡ(ü'\0') */
+extern void* CSV_memory( void* pMem, char* pcCSV, int iParam/*Хȿ*/ ); /* Τޤޥԡ */
+/* ...ޥؿ򼫺ơѤ뤳Ȥǽ (^^)v */
+
+
 #endif	/*LIBNET_H*/
 
 
